package dnddemo3;
import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.DefaultListModel;
import javax.swing.DropMode;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.TransferHandler;


public class DnDDemo3 {

    private static void constructGUI() {

        // The first panel, containing a JTextField
        JPanel north = new JPanel();
        north.add(new JLabel("Drag from here:"));
        JTextField field = new JTextField(10);
        field.setDragEnabled(true);     // enabled drag and drop
        north.add(field);

        // The second panel in the middle, containing
        // list that will become a drop target
        final DefaultListModel listModel = new DefaultListModel();
        listModel.addElement("first");
        listModel.addElement("second");
        final JList list = new JList(listModel);
        list.setDragEnabled(true);      // enabled drag and drop

        // Set customized transfer handler 
        list.setTransferHandler(new TransferHandler() {
            public boolean canImport(TransferHandler.TransferSupport support) {
                // Return false if string DataFlavor is not supported
                if (! support.isDataFlavorSupported(
                        DataFlavor.stringFlavor)) {
                    return false;
                }
                JList.DropLocation dl = (JList.DropLocation) 
                        support.getDropLocation();
                if (dl.getIndex() == -1) {
                    return false;
                } else {
                    return true;
                }
            }

            public boolean importData(TransferHandler.TransferSupport support) {
                if (! canImport(support)) {
                    return false;
                }    
                
                // get Transferable
                Transferable transferable = support.getTransferable();
                String data;
                try {
                    data = (String) transferable.getTransferData(
                            DataFlavor.stringFlavor);
                } catch (Exception e) {
                    return false;
                }

                JList.DropLocation dl = (JList.DropLocation) 
                        support.getDropLocation();
                int index = dl.getIndex();
                if (dl.isInsert()) {
                    listModel.add(index, data);
                } else {
                    listModel.set(index, data);
                }

                // Scroll to display the element that was dropped
                Rectangle r = list.getCellBounds(index, index);
                list.scrollRectToVisible(r);
                return true;
            }
        });
        JScrollPane center = new JScrollPane();
        center.setViewportView(list);

        // the third panel containing a combobox to change the JList's drop mode
        DropMode[] modes = new DropMode[] {DropMode.USE_SELECTION, 
                DropMode.ON, DropMode.INSERT, DropMode.ON_OR_INSERT};
        final JComboBox combo = new JComboBox(modes);
        combo.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                DropMode mode = (DropMode) combo.getSelectedItem();
                list.setDropMode(mode);
            }
        });
        JPanel south = new JPanel();
        south.add(new JLabel("Drop mode:"));
        south.add(combo);

        JPanel cp = new JPanel();
        cp.setLayout(new BorderLayout());
        cp.add(north, BorderLayout.NORTH);
        cp.add(south, BorderLayout.SOUTH);
        cp.add(center, BorderLayout.CENTER);
        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setTitle("JList's Drag & Drop");
        frame.setContentPane(cp);
        frame.pack();
        frame.setVisible(true);
    }
    
    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                constructGUI();
            }
        });
    }
}
